@@ -142,7 +142,7 @@ static void stress_fiemap_ioctl(
 
 		fiemap = (struct fiemap *)calloc(1, sizeof(*fiemap));
 		if (UNLIKELY(!fiemap)) {
-			pr_err("Out of memory allocating fiemap\n");
+			pr_err("out of memory allocating fiemap\n");
 			break;
 		}
 		fiemap->fm_length = ~0UL;
@@ -259,7 +259,7 @@ static int stress_fiemap(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
-	s_pids = stress_s_pids_mmap(MAX_FIEMAP_PROCS);
+	s_pids = stress_sync_s_pids_mmap(MAX_FIEMAP_PROCS);
 	if (s_pids == MAP_FAILED) {
 		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, MAX_FIEMAP_PROCS);
 		stress_lock_destroy(counter_lock);
@@ -272,7 +272,7 @@ static int stress_fiemap(stress_args_t *args)
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			fiemap_bytes = MIN_FIEMAP_SIZE;
 	}
-	fiemap_bytes /= args->num_instances;
+	fiemap_bytes /= args->instances;
 	if (fiemap_bytes < MIN_FIEMAP_SIZE)
 		fiemap_bytes = MIN_FIEMAP_SIZE;
 
@@ -339,7 +339,7 @@ dir_clean:
 clean:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)stress_s_pids_munmap(s_pids, MAX_FIEMAP_PROCS);
+	(void)stress_sync_s_pids_munmap(s_pids, MAX_FIEMAP_PROCS);
 	stress_lock_destroy(counter_lock);
 
 	return rc;
