@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,7 +36,7 @@ static const stress_help_t help[] = {
 };
 
 /* paths we should never stat */
-static const char *blocklist[] = {
+static const char * const blocklist[] = {
 	"/dev/watchdog"
 };
 
@@ -71,11 +71,6 @@ typedef struct ctxt {
 	int bad_fd;			/* bad/invalid fd */
 } stress_fstat_context_t;
 
-static int stress_set_fstat_dir(const char *opt)
-{
-	return stress_set_setting("fstat-dir", TYPE_ID_STR, opt);
-}
-
 /*
  *  handle_fstat_sigalrm()
  *      catch SIGALRM
@@ -215,7 +210,6 @@ static int stress_fstat_helper(const stress_fstat_context_t *ctxt)
  */
 static void *stress_fstat_thread(void *ptr)
 {
-	static void *nowt = NULL;
 	stress_fstat_pthread_info_t *pthread_info = (stress_fstat_pthread_info_t *)ptr;
 	const stress_fstat_context_t *ctxt = pthread_info->ctxt;
 
@@ -228,11 +222,13 @@ static void *stress_fstat_thread(void *ptr)
 #if !defined(__APPLE__)
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 #endif
-	while (keep_running && stress_continue_flag()) {
+	stress_random_small_sleep();
+
+	while (LIKELY(keep_running && stress_continue_flag())) {
 		size_t i;
 
 		for (i = 0; i < FSTAT_LOOPS; i++) {
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 			if (stress_fstat_helper(ctxt) < 0) {
 				pthread_info->pthread_ret = -1;
@@ -242,7 +238,7 @@ static void *stress_fstat_thread(void *ptr)
 		(void)shim_sched_yield();
 	}
 
-	return &nowt;
+	return &g_nowt;
 }
 #endif
 
@@ -277,7 +273,7 @@ static int stress_fstat_threads(stress_args_t *args, stress_stat_info_t *si, con
 	}
 #endif
 	for (i = 0; i < FSTAT_LOOPS; i++) {
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		if (stress_fstat_helper(&ctxt) < 0) {
 			rc = -1;
@@ -306,7 +302,7 @@ static int stress_fstat(stress_args_t *args)
 {
 	stress_stat_info_t *si;
 	static stress_stat_info_t *stat_info;
-	struct dirent *d;
+	const struct dirent *d;
 	NOCLOBBER int ret = EXIT_FAILURE;
 	bool stat_some;
 	const uid_t euid = geteuid();
@@ -319,7 +315,7 @@ static int stress_fstat(stress_args_t *args)
 		return EXIT_FAILURE;
 
 	if ((dp = opendir(fstat_dir)) == NULL) {
-		pr_err("%s: opendir on %s failed: errno=%d: (%s)\n",
+		pr_err("%s: opendir on %s failed, errno=%d: (%s)\n",
 			args->name, fstat_dir, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -328,7 +324,7 @@ static int stress_fstat(stress_args_t *args)
 	while ((d = readdir(dp)) != NULL) {
 		char path[PATH_MAX];
 
-		if (!stress_continue_flag()) {
+		if (UNLIKELY(!stress_continue_flag())) {
 			ret = EXIT_SUCCESS;
 			(void)closedir(dp);
 			goto free_cache;
@@ -337,7 +333,7 @@ static int stress_fstat(stress_args_t *args)
 		(void)stress_mk_filename(path, sizeof(path), fstat_dir, d->d_name);
 		if (do_not_stat(path))
 			continue;
-		if ((si = calloc(1, sizeof(*si))) == NULL) {
+		if ((si = (stress_stat_info_t *)calloc(1, sizeof(*si))) == NULL) {
 			pr_err("%s: out of memory\n", args->name);
 			(void)closedir(dp);
 			goto free_cache;
@@ -356,13 +352,15 @@ static int stress_fstat(stress_args_t *args)
 	(void)closedir(dp);
 
 	(void)sigfillset(&set);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		stat_some = false;
 
-		for (si = stat_info; si && stress_continue_flag(); si = si->next) {
-			if (!stress_continue(args))
+		for (si = stat_info; LIKELY(si && stress_continue_flag()); si = si->next) {
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 			if (si->ignore == IGNORE_ALL)
 				continue;
@@ -390,15 +388,15 @@ free_cache:
 	return ret;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_fstat_dir,	stress_set_fstat_dir },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_fstat_dir, "fstat-dir", TYPE_ID_STR, 0, 0, NULL },
+	END_OPT,
 };
 
-stressor_info_t stress_fstat_info = {
+const stressor_info_t stress_fstat_info = {
 	.stressor = stress_fstat,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
