@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyrignt (C) 2021-2023 Colin Ian King.
+ * Copyrignt (C) 2021-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,8 +18,12 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 #include "core-put.h"
 
+#include <sys/ioctl.h>
+
 #if defined(HAVE_LINUX_FS_H)
 #include <linux/fs.h>
 #endif
@@ -31,7 +35,7 @@
 #define MAX_IOMIX_BYTES		(MAX_FILE_LIMIT)
 #define DEFAULT_IOMIX_BYTES	(1 * GB)
 
-typedef void (*stress_iomix_func)(const stress_args_t *args, const int fd, const char *fs_type, const off_t iomix_bytes);
+typedef void (*stress_iomix_func)(stress_args_t *args, const int fd, const char *fs_type, const off_t iomix_bytes);
 
 static const stress_help_t help[] = {
 	{ NULL,	"iomix N",	 "start N workers that have a mix of I/O operations" },
@@ -42,16 +46,6 @@ static const stress_help_t help[] = {
 
 static void *counter_lock;
 
-static int stress_set_iomix_bytes(const char *opt)
-{
-	off_t iomix_bytes;
-
-	iomix_bytes = (off_t)stress_get_uint64_byte_filesystem(opt, 1);
-	stress_check_range_bytes("iomix-bytes", (uint64_t)iomix_bytes,
-		MIN_IOMIX_BYTES, MAX_IOMIX_BYTES);
-	return stress_set_setting("iomix-bytes", TYPE_ID_OFF_T, &iomix_bytes);
-}
-
 /*
  *  stress_iomix_rnd_offset()
  *	generate a random offset between 0..max-1
@@ -103,7 +97,7 @@ static void stress_iomix_fsync_min_1Hz(const int fd)
 	if (time_last <= 0.0)
 		time_last = stress_time_now() + 1.0;
 
-	if (counter++ >= counter_max) {
+	if (UNLIKELY(counter++ >= counter_max)) {
 		const double now = stress_time_now();
 		const double delta = now - time_last;
 
@@ -135,7 +129,7 @@ static void stress_iomix_fsync_min_1Hz(const int fd)
  *	bursty sequential writes
  */
 static void stress_iomix_wr_seq_bursts(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -147,7 +141,9 @@ static void stress_iomix_wr_seq_bursts(
 
 		posn = stress_iomix_rnd_offset(iomix_bytes);
 		ret = lseek(fd, posn, SEEK_SET);
-		if (ret == (off_t)-1) {
+		if (UNLIKELY(ret == (off_t)-1)) {
+			if (errno == EINTR)
+				return;
 			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			return;
@@ -167,8 +163,10 @@ static void stress_iomix_wr_seq_bursts(
 			stress_rndbuf(buffer, len);
 
 			rc = write(fd, buffer, len);
-			if (rc < 0) {
-				if (errno != EPERM) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
+				if ((errno != EPERM) && (errno != ENOSPC)) {
 					pr_fail("%s: write failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno), fs_type);
 					return;
@@ -176,12 +174,12 @@ static void stress_iomix_wr_seq_bursts(
 			}
 			posn += rc;
 
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (!stress_bogo_inc_lock(args, counter_lock, true))
 				return;
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-		shim_usleep(stress_mwc32modn(1000000));
-	} while (inc_counter_lock(args, counter_lock, false));
+		(void)shim_usleep(stress_mwc32modn(1000000));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -189,7 +187,7 @@ static void stress_iomix_wr_seq_bursts(
  *	bursty random writes
  */
 static void stress_iomix_wr_rnd_bursts(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -212,7 +210,9 @@ static void stress_iomix_wr_rnd_bursts(
 
 			posn = stress_iomix_rnd_offset(iomix_bytes);
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret == (off_t)-1) {
+			if (UNLIKELY(ret == (off_t)-1)) {
+				if (errno == EINTR)
+					return;
 				pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
 				return;
@@ -220,19 +220,21 @@ static void stress_iomix_wr_rnd_bursts(
 
 			stress_rndbuf(buffer, len);
 			rc = write(fd, buffer, len);
-			if (rc < 0) {
-				if (errno != EPERM) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
+				if ((errno != EPERM) && (errno != ENOSPC)) {
 					pr_fail("%s: write failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno), fs_type);
 					return;
 				}
 			}
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-		shim_usleep(stress_mwc32modn(2000000));
-	} while (inc_counter_lock(args, counter_lock, false));
+		(void)shim_usleep(stress_mwc32modn(2000000));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -240,7 +242,7 @@ static void stress_iomix_wr_rnd_bursts(
  *	slow sequential writes
  */
 static void stress_iomix_wr_seq_slow(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -249,7 +251,9 @@ static void stress_iomix_wr_seq_slow(
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret == (off_t)-1) {
+		if (UNLIKELY(ret == (off_t)-1)) {
+			if (errno == EINTR)
+				return;
 			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			return;
@@ -269,8 +273,10 @@ static void stress_iomix_wr_seq_slow(
 			stress_rndbuf(buffer, len);
 
 			rc = write(fd, buffer, len);
-			if (rc < 0) {
-				if (errno != EPERM) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
+				if ((errno != EPERM) && (errno != ENOSPC)) {
 					pr_fail("%s: write failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno), fs_type);
 					return;
@@ -278,11 +284,11 @@ static void stress_iomix_wr_seq_slow(
 			}
 			(void)shim_usleep(250000);
 			posn += rc;
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -290,7 +296,7 @@ static void stress_iomix_wr_seq_slow(
  *	bursty sequential reads
  */
 static void stress_iomix_rd_seq_bursts(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -302,7 +308,9 @@ static void stress_iomix_rd_seq_bursts(
 
 		posn = stress_iomix_rnd_offset(iomix_bytes);
 		ret = lseek(fd, posn, SEEK_SET);
-		if (ret == (off_t)-1) {
+		if (UNLIKELY(ret == (off_t)-1)) {
+			if (errno == EINTR)
+				return;
 			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			return;
@@ -321,20 +329,22 @@ static void stress_iomix_rd_seq_bursts(
 			const size_t len = 1 + (stress_mwc32() & (sizeof(buffer) - 1));
 
 			rc = read(fd, buffer, len);
-			if (rc < 0) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
 				pr_fail("%s: read failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 			posn += rc;
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 
 			/* Add some unhelpful advice */
 			stress_iomix_fadvise_random_dontneed(fd, posn, 4096);
 		}
-		shim_usleep(stress_mwc32modn(1000000));
-	} while (inc_counter_lock(args, counter_lock, false));
+		(void)shim_usleep(stress_mwc32modn(1000000));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -342,7 +352,7 @@ static void stress_iomix_rd_seq_bursts(
  *	bursty random reads
  */
 static void stress_iomix_rd_rnd_bursts(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -363,23 +373,27 @@ static void stress_iomix_rd_rnd_bursts(
 			stress_iomix_fadvise_random_dontneed(fd, posn, (ssize_t)len);
 
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret == (off_t)-1) {
+			if (UNLIKELY(ret == (off_t)-1)) {
+				if (errno == EINTR)
+					return;
 				pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 
 			rc = read(fd, buffer, len);
-			if (rc < 0) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
 				pr_fail("%s: read failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 		}
-		shim_usleep(3000000);
-	} while (inc_counter_lock(args, counter_lock, false));
+		(void)shim_usleep(3000000);
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -387,7 +401,7 @@ static void stress_iomix_rd_rnd_bursts(
  *	slow sequential reads
  */
 static void stress_iomix_rd_seq_slow(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -396,7 +410,9 @@ static void stress_iomix_rd_seq_slow(
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret == (off_t)-1) {
+		if (UNLIKELY(ret == (off_t)-1)) {
+			if (errno == EINTR)
+				return;
 			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			return;
@@ -417,18 +433,20 @@ static void stress_iomix_rd_seq_slow(
 			stress_iomix_fadvise_random_dontneed(fd, posn, (ssize_t)len);
 
 			rc = read(fd, buffer, len);
-			if (rc < 0) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
 				pr_fail("%s: read failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 			(void)shim_usleep(333333);
 			posn += rc;
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -436,7 +454,7 @@ static void stress_iomix_rd_seq_slow(
  *	file syncs
  */
 static void stress_iomix_sync(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -445,20 +463,20 @@ static void stress_iomix_sync(
 
 	do {
 		(void)shim_fsync(fd);
-		if (!inc_counter_lock(args, counter_lock, true))
+		if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 			break;
-		shim_usleep(stress_mwc32modn(4000000));
-		if (!inc_counter_lock(args, counter_lock, false))
+		(void)shim_usleep(stress_mwc32modn(4000000));
+		if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 			break;
 
 #if defined(HAVE_FDATASYNC)
 		(void)shim_fdatasync(fd);
 		/* Exercise illegal fdatasync */
 		(void)shim_fdatasync(-1);
-		if (!inc_counter_lock(args, counter_lock, false))
+		if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 			break;
-		shim_usleep(stress_mwc32modn(4000000));
-		if (!inc_counter_lock(args, counter_lock, false))
+		(void)shim_usleep(stress_mwc32modn(4000000));
+		if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 			break;
 #else
 		UNEXPECTED
@@ -472,15 +490,15 @@ static void stress_iomix_sync(
 				SYNC_FILE_RANGE_WRITE);
 			stress_iomix_fadvise_random_dontneed(fd, posn, 65536);
 
-			if (!inc_counter_lock(args, counter_lock, false))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 				break;
-			shim_usleep(stress_mwc32modn(4000000));
+			(void)shim_usleep(stress_mwc32modn(4000000));
 		}
 #else
 		(void)iomix_bytes;
 		UNEXPECTED
 #endif
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 #if defined(HAVE_POSIX_FADVISE) &&	\
@@ -490,7 +508,7 @@ static void stress_iomix_sync(
  *	bad fadvise hints
  */
 static void stress_iomix_bad_advise(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -504,7 +522,7 @@ static void stress_iomix_bad_advise(
 		(void)shim_usleep(100000);
 		(void)posix_fadvise(fd, posn, 65536, POSIX_FADV_NORMAL);
 		(void)shim_usleep(100000);
-	} while (inc_counter_lock(args, counter_lock, true));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 }
 #endif
 
@@ -513,7 +531,7 @@ static void stress_iomix_bad_advise(
  *	random memory mapped read/writes
  */
 static void stress_iomix_rd_wr_mmap(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -536,15 +554,11 @@ static void stress_iomix_rd_wr_mmap(
 					PROT_READ | PROT_WRITE, flags, fd, posn);
 		}
 		for (i = 0; i < SIZEOF_ARRAY(mmaps); i++) {
-			if (mmaps[i] != MAP_FAILED) {
-				size_t j;
-				uint64_t sum = 0;
-				uint8_t *buffer = (uint8_t *)mmaps[i];
+			if (LIKELY(mmaps[i] != MAP_FAILED)) {
+				const uint8_t *buffer = (uint8_t *)mmaps[i];
 
 				/* Force page data to be read */
-				for (j = 0; j < page_size; j++)
-					sum += buffer[j];
-				stress_uint64_put(sum);
+				stress_uint8_put(buffer[0]);
 
 				stress_rndbuf(mmaps[i], page_size);
 				(void)shim_msync(mmaps[i], page_size,
@@ -553,10 +567,10 @@ static void stress_iomix_rd_wr_mmap(
 		}
 		(void)shim_usleep(100000);
 		for (i = 0; i < SIZEOF_ARRAY(mmaps); i++) {
-			if (mmaps[i] != MAP_FAILED)
+			if (LIKELY(mmaps[i] != MAP_FAILED))
 				(void)munmap(mmaps[i], page_size);
 		}
-	} while (inc_counter_lock(args, counter_lock, true));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 }
 
 /*
@@ -564,7 +578,7 @@ static void stress_iomix_rd_wr_mmap(
  *	lots of small 1 byte writes
  */
 static void stress_iomix_wr_bytes(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -573,7 +587,9 @@ static void stress_iomix_wr_bytes(
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret == (off_t)-1) {
+		if (UNLIKELY(ret == (off_t)-1)) {
+			if (errno == EINTR)
+				return;
 			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			return;
@@ -583,19 +599,21 @@ static void stress_iomix_wr_bytes(
 			ssize_t rc;
 
 			rc = write(fd, buffer, sizeof(buffer));
-			if (rc < 0) {
-				if (errno != EPERM) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
+				if ((errno != EPERM) && (errno != ENOSPC)) {
 					pr_fail("%s: write failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno), fs_type);
 					return;
 			}	}
 			(void)shim_usleep(1000);
 			posn += rc;
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -603,7 +621,7 @@ static void stress_iomix_wr_bytes(
  *	lots of small 1 byte writes in reverse order
  */
 static void stress_iomix_wr_rev_bytes(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -612,7 +630,9 @@ static void stress_iomix_wr_rev_bytes(
 		off_t ret, posn = iomix_bytes;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret == (off_t)-1) {
+		if (UNLIKELY(ret == (off_t)-1)) {
+			if (errno == EINTR)
+				return;
 			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			return;
@@ -622,19 +642,21 @@ static void stress_iomix_wr_rev_bytes(
 			ssize_t rc;
 
 			rc = write(fd, buffer, sizeof(buffer));
-			if (rc < 0) {
-				if (errno != EPERM) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
+				if ((errno != EPERM) && (errno != ENOSPC)) {
 					pr_fail("%s: write failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno), fs_type);
 					return;
 			}	}
 			(void)shim_usleep(1000);
 			posn--;
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 /*
@@ -642,31 +664,36 @@ static void stress_iomix_wr_rev_bytes(
  *	lots of small 1 byte reads
  */
 static void stress_iomix_rd_bytes(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
-		off_t ret, posn = iomix_bytes;
+		off_t posn = iomix_bytes;
 
 		while (posn != 0) {
 			char buffer[1];
 			ssize_t rc;
+			off_t ret;
 
 			/* Add some unhelpful advice */
 			stress_iomix_fadvise_random_dontneed(fd, posn, sizeof(buffer));
 
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret == (off_t)-1) {
+			if (UNLIKELY(ret == (off_t)-1)) {
+				if (errno == EINTR)
+					return;
 				pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 
 			rc = read(fd, buffer, sizeof(buffer));
-			if (rc < 0) {
-				if (errno != EPERM) {
+			if (UNLIKELY(rc < 0)) {
+				if (errno == EINTR)
+					break;
+				if ((errno != EPERM) && (errno != ENOSPC)) {
 					pr_fail("%s: write failed, errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno), fs_type);
 					return;
@@ -674,10 +701,10 @@ static void stress_iomix_rd_bytes(
 			}
 			(void)shim_usleep(1000);
 			posn--;
-			if (!inc_counter_lock(args, counter_lock, true))
+			if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 				return;
 		}
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 
 #if defined(__linux__)
@@ -698,29 +725,29 @@ static void stress_iomix_rd_bytes(
  *	attempt to set and unset a file based inode flag
  */
 static void stress_iomix_inode_ioctl(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const int flag,
 	bool *ok)
 {
+#if defined(FS_IOC_GETFLAGS)
 	int ret, attr;
 
-	if (!keep_stressing(args))
+	if (UNLIKELY(!stress_continue(args)))
 		return;
 
-#if defined(FS_IOC_GETFLAGS)
 	ret = ioctl(fd, FS_IOC_GETFLAGS, &attr);
-	if (ret < 0)
+	if (UNLIKELY(ret < 0))
 		return;
 #if defined(FS_IOC_SETFLAGS)
 	attr |= flag;
 	ret = ioctl(fd, FS_IOC_SETFLAGS, &attr);
-	if (ret < 0)
+	if (UNLIKELY(ret < 0))
 		return;
 
 	attr &= ~flag;
 	ret = ioctl(fd, FS_IOC_SETFLAGS, &attr);
-	if (ret < 0)
+	if (UNLIKELY(ret < 0))
 		return;
 #else
 	UNEXPECTED
@@ -737,7 +764,7 @@ static void stress_iomix_inode_ioctl(
  *	twiddle various inode flags
  */
 static void stress_iomix_inode_flags(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -807,7 +834,7 @@ static void stress_iomix_inode_flags(
 		if (!ok)
 			_exit(EXIT_SUCCESS);
 		stress_iomix_fsync_min_1Hz(fd);
-	} while (inc_counter_lock(args, counter_lock, true));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 }
 #endif
 
@@ -817,7 +844,7 @@ static void stress_iomix_inode_flags(
  *	occasional file cache dropping
  */
 static void stress_iomix_drop_caches(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -828,22 +855,22 @@ static void stress_iomix_drop_caches(
 
 	do {
 		(void)sync();
-		if (system_write("/proc/sys/vm/drop_caches", "1", 1) < 0)
+		if (stress_system_write("/proc/sys/vm/drop_caches", "1", 1) < 0)
 			(void)pause();
 		(void)sleep(5);
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			return;
 		(void)sync();
-		if (system_write("/proc/sys/vm/drop_caches", "2", 1) < 0)
+		if (stress_system_write("/proc/sys/vm/drop_caches", "2", 1) < 0)
 			(void)pause();
 		(void)sleep(5);
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			return;
 		(void)sync();
-		if (system_write("/proc/sys/vm/drop_caches", "3", 1) < 0)
+		if (stress_system_write("/proc/sys/vm/drop_caches", "3", 1) < 0)
 			(void)pause();
 		(void)sleep(5);
-	} while (inc_counter_lock(args, counter_lock, true));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 }
 #endif
 
@@ -853,7 +880,7 @@ static void stress_iomix_drop_caches(
  *	lots of copies with copy_file_range
  */
 static void stress_iomix_copy_file_range(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -861,19 +888,19 @@ static void stress_iomix_copy_file_range(
 	(void)fs_type;
 
 	do {
-		off_t from = stress_iomix_rnd_offset(iomix_bytes);
-		off_t to = stress_iomix_rnd_offset(iomix_bytes);
+		shim_off64_t from = (shim_off64_t)stress_iomix_rnd_offset(iomix_bytes);
+		shim_off64_t to = (shim_off64_t)stress_iomix_rnd_offset(iomix_bytes);
 		const size_t size = stress_mwc16();
 
-		VOID_RET(ssize_t, copy_file_range(fd, &from, fd, &to, size, 0));
-		VOID_RET(ssize_t, copy_file_range(fd, &to, fd, &from, size, 0));
+		VOID_RET(ssize_t, shim_copy_file_range(fd, &from, fd, &to, size, 0));
+		VOID_RET(ssize_t, shim_copy_file_range(fd, &to, fd, &from, size, 0));
 
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			return;
 		stress_iomix_fsync_min_1Hz(fd);
 
-		shim_usleep(stress_mwc32modn(100000));
-	} while (inc_counter_lock(args, counter_lock, true));
+		(void)shim_usleep(stress_mwc32modn(100000));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 }
 #endif
 
@@ -884,7 +911,7 @@ static void stress_iomix_copy_file_range(
  *	lots of copies with copy_file_range
  */
 static void stress_iomix_sendfile(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t iomix_bytes)
@@ -898,19 +925,128 @@ static void stress_iomix_sendfile(
 		const size_t size = stress_mwc16();
 
 		ret = lseek(fd, to, SEEK_SET);
-		if (ret != (off_t)-1) {
+		if (UNLIKELY(ret != (off_t)-1)) {
 			ssize_t sret;
 
 			sret = sendfile(fd, fd, &from, size);
 			(void)sret;
 		}
 
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			return;
 		stress_iomix_fsync_min_1Hz(fd);
 
-		shim_usleep(stress_mwc32modn(130000));
-	} while (inc_counter_lock(args, counter_lock, true));
+		(void)shim_usleep(stress_mwc32modn(130000));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
+}
+#endif
+
+#if defined(__linux__) &&	\
+    defined(__NR_cachestat)
+struct shim_cachestat_range {
+	uint64_t off;
+	uint64_t len;
+};
+
+struct shim_cachestat {
+	uint64_t nr_cache;
+	uint64_t nr_dirty;
+	uint64_t nr_writeback;
+	uint64_t nr_evicted;
+	uint64_t nr_recently_evicted;
+};
+
+/*
+ *  shim_cachestat
+ *	wrapper for cachestat system call
+ */
+static inline int shim_cachestat(
+	int fd,
+	struct shim_cachestat_range *cstat_range,
+	struct shim_cachestat *cstat,
+	unsigned int flags)
+{
+	return (int)syscall(__NR_cachestat,
+			(unsigned long int)fd,
+			(unsigned long int)cstat_range,
+			(unsigned long int)cstat,
+			(unsigned long int)flags);
+}
+
+/*
+ *  stress_iomix_cachestat()
+ *	various periodic cache statistics calls (linux only)
+ */
+static void stress_iomix_cachestat(
+	stress_args_t *args,
+	const int fd,
+	const char *fs_type,
+	const off_t iomix_bytes)
+{
+	(void)fs_type;
+	(void)iomix_bytes;
+
+	do {
+		struct stat buf;
+
+		if (shim_fstat(fd, &buf) == 0) {
+			struct shim_cachestat_range cstat_range;
+			struct shim_cachestat cstat;
+
+			cstat_range.off = (uint64_t)0ULL;
+			cstat_range.len = (uint64_t)buf.st_size;
+			VOID_RET(int, shim_cachestat(fd, &cstat_range, &cstat, 0));
+
+			cstat_range.off = (uint64_t)0ULL;
+			cstat_range.len = (uint64_t)512;
+			VOID_RET(int, shim_cachestat(fd, &cstat_range, &cstat, 0));
+
+			cstat_range.off = (uint64_t)buf.st_size;
+			cstat_range.len = (uint64_t)512;
+			VOID_RET(int, shim_cachestat(fd, &cstat_range, &cstat, 0));
+
+			cstat_range.off = (uint64_t)0ULL;
+			cstat_range.len = (uint64_t)0ULL;
+			VOID_RET(int, shim_cachestat(fd, &cstat_range, &cstat, 0));
+
+			cstat_range.off = (uint64_t)0ULL;
+			cstat_range.len = (uint64_t)iomix_bytes;
+			VOID_RET(int, shim_cachestat(fd, &cstat_range, &cstat, 0));
+
+			/* exercise invalid flags */
+			cstat_range.off = (uint64_t)0ULL;
+			cstat_range.len = (uint64_t)buf.st_size;
+			VOID_RET(int, shim_cachestat(fd, &cstat_range, &cstat, ~0));
+
+			/* exercise invalid fd */
+			cstat_range.off = (uint64_t)0ULL;
+			cstat_range.len = (uint64_t)buf.st_size;
+			VOID_RET(int, shim_cachestat(100000, &cstat_range, &cstat, 0));
+		}
+		(void)shim_usleep(50000);
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
+}
+#endif
+
+#if defined(HAVE_READAHEAD)
+static void stress_iomix_readahead(
+	stress_args_t *args,
+	const int fd,
+	const char *fs_type,
+	const off_t iomix_bytes)
+{
+	(void)fs_type;
+
+	do {
+		const off_t offset = stress_iomix_rnd_offset(iomix_bytes);
+		const size_t len = 512 * stress_mwc8modn(16);
+
+		VOID_RET(int, readahead(fd, offset, len));
+
+		(void)shim_usleep(stress_mwc32modn(2000000));
+		if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
+			return;
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 }
 #endif
 
@@ -944,32 +1080,51 @@ static stress_iomix_func iomix_funcs[] = {
     defined(HAVE_SENDFILE)
 	stress_iomix_sendfile,
 #endif
+#if defined(__linux__) &&	\
+    defined(__NR_cachestat)
+	stress_iomix_cachestat,
+#endif
+#if defined(HAVE_READAHEAD)
+	stress_iomix_readahead,
+#endif
 };
 
+#define MAX_IOMIX_PROCS	(SIZEOF_ARRAY(iomix_funcs))
+
 /*
  *  stress_iomix
  *	stress I/O via random mix of io ops
  */
-static int stress_iomix(const stress_args_t *args)
+static int stress_iomix(stress_args_t *args)
 {
 	int fd, ret;
 	char filename[PATH_MAX];
 	off_t iomix_bytes = DEFAULT_IOMIX_BYTES;
 	const size_t page_size = args->page_size;
 	size_t i;
-	int pids[SIZEOF_ARRAY(iomix_funcs)];
+	stress_pid_t *s_pids, *s_pids_head = NULL;
 	const char *fs_type;
 	int oflags = O_CREAT | O_RDWR;
-	const pid_t parent = getpid();
+	bool iomix_bytes_shrunk = false;
+
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
+	s_pids = stress_sync_s_pids_mmap(MAX_IOMIX_PROCS);
+	if (s_pids == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %zu PIDs, skipping stressor\n", args->name, MAX_IOMIX_PROCS);
+		return EXIT_NO_RESOURCE;
+	}
 
 #if defined(O_SYNC)
 	oflags |= O_SYNC;
 #endif
 
-	counter_lock = stress_lock_create();
+	counter_lock = stress_lock_create("counter");
 	if (!counter_lock) {
 		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
-		return EXIT_NO_RESOURCE;
+		ret = EXIT_NO_RESOURCE;
+		goto tidy_s_pids;
 	}
 
 	if (!stress_get_setting("iomix-bytes", &iomix_bytes)) {
@@ -978,7 +1133,7 @@ static int stress_iomix(const stress_args_t *args)
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			iomix_bytes = MIN_IOMIX_BYTES;
 	}
-	iomix_bytes /= args->num_instances;
+	iomix_bytes /= args->instances;
 	if (iomix_bytes < (off_t)MIN_IOMIX_BYTES)
 		iomix_bytes = (off_t)MIN_IOMIX_BYTES;
 	if (iomix_bytes < (off_t)page_size)
@@ -998,62 +1153,75 @@ static int stress_iomix(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto lock_destroy;
 	}
-	fs_type = stress_fs_type(filename);
+	fs_type = stress_get_fs_type(filename);
 	(void)shim_unlink(filename);
 
+	do {
 #if defined(FALLOC_FL_ZERO_RANGE)
-	ret = shim_fallocate(fd, FALLOC_FL_ZERO_RANGE, 0, iomix_bytes);
+		ret = shim_fallocate(fd, FALLOC_FL_ZERO_RANGE, 0, iomix_bytes);
 #else
-	ret = shim_fallocate(fd, 0, 0, iomix_bytes);
+		ret = shim_fallocate(fd, 0, 0, iomix_bytes);
 #endif
-	if (ret < 0) {
-		if (errno == ENOSPC) {
-			ret = EXIT_NO_RESOURCE;
-		} else {
-			ret = EXIT_FAILURE;
-			pr_fail("%s: fallocate failed, errno=%d (%s)%s\n",
-				args->name, errno, strerror(errno), fs_type);
+		if (UNLIKELY(ret < 0)) {
+			switch (errno) {
+			case EFBIG:
+			case ENOSPC:
+				if (iomix_bytes > (off_t)MIN_IOMIX_BYTES) {
+					iomix_bytes >>= 1;
+					iomix_bytes_shrunk = true;
+				} else {
+					pr_fail("%s: fallocate failed, no free space, errno=%d (%s)%s, skipping stressor\n",
+						args->name, errno, strerror(errno), fs_type);
+					ret = EXIT_NO_RESOURCE;
+					goto tidy;
+				}
+				break;
+			default:
+				pr_fail("%s: fallocate failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
+				ret = EXIT_FAILURE;
+				goto tidy;
+			}
 		}
-		goto tidy;
-	}
+	} while ((ret < 0) && stress_continue(args));
 
-	(void)memset(pids, 0, sizeof(pids));
+	if (iomix_bytes_shrunk)
+		pr_inf("%s: file size too large for file system, reducing file size to %" PRIdMAX " MB\n",
+			args->name, (intmax_t)iomix_bytes >> 20);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_file_rw_hint_short(fd);
 
-	for (i = 0; i < SIZEOF_ARRAY(iomix_funcs); i++) {
-		pids[i] = fork();
-		if (pids[i] < 0) {
+	for (i = 0; i < MAX_IOMIX_PROCS; i++) {
+		stress_sync_start_init(&s_pids[i]);
+
+		s_pids[i].pid = fork();
+		if (s_pids[i].pid < 0) {
 			goto reap;
-		} else if (pids[i] == 0) {
+		} else if (s_pids[i].pid == 0) {
+			s_pids[i].pid = getpid();
+			stress_sync_start_wait_s_pid(&s_pids[i]);
+
 			/* Child */
 			(void)sched_settings_apply(true);
 			iomix_funcs[i](args, fd, fs_type, iomix_bytes);
-			(void)kill(parent, SIGALRM);
 			_exit(EXIT_SUCCESS);
+		} else {
+			stress_sync_start_s_pid_list_add(&s_pids_head, &s_pids[i]);
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_sync_start_cont_list(s_pids_head);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	do {
 		pause();
-	} while (inc_counter_lock(args, counter_lock, false));
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
 
 	ret = EXIT_SUCCESS;
 reap:
-	for (i = 0; i < SIZEOF_ARRAY(iomix_funcs); i++) {
-		if (pids[i]) {
-			(void)kill(pids[i], SIGALRM);
-			(void)kill(pids[i], SIGKILL);
-		}
-	}
-	for (i = 0; i < SIZEOF_ARRAY(iomix_funcs); i++) {
-		if (pids[i]) {
-			int status;
-
-			(void)shim_waitpid(pids[i], &status, 0);
-		}
-	}
-
+	stress_kill_and_wait_many(args, s_pids, MAX_IOMIX_PROCS, SIGALRM, true);
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fd);
@@ -1061,19 +1229,21 @@ tidy:
 lock_destroy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)stress_lock_destroy(counter_lock);
+tidy_s_pids:
+	(void)stress_sync_s_pids_munmap(s_pids, MAX_IOMIX_PROCS);
 
 	return ret;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_iomix_bytes,	stress_set_iomix_bytes },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_iomix_bytes, "iomix-bytes", TYPE_ID_OFF_T, MIN_IOMIX_BYTES, MAX_IOMIX_BYTES, NULL },
+	END_OPT,
 };
 
-stressor_info_t stress_iomix_info = {
+const stressor_info_t stress_iomix_info = {
 	.stressor = stress_iomix,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
