@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King.
+ * Copyright (C) 2021-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@ static const stress_help_t help[] = {
  *  stress on sched_kill()
  *	stress system by rapid kills
  */
-static int stress_kill(const stress_args_t *args)
+static int stress_kill(stress_args_t *args)
 {
 	uint64_t udelay = 5000;
 	pid_t pid;
@@ -42,16 +42,21 @@ static int stress_kill(const stress_args_t *args)
 
 	pid = fork();
 	if (pid == 0) {
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 		VOID_RET(int, stress_sighandler(args->name, SIGUSR1, stress_sighandler_nop, NULL));
 
-		while (keep_stressing(args)) {
+		while (stress_continue(args)) {
 			if (kill(ppid, 0) < 0)
 				break;
 			pause();
 		}
+		stress_set_proc_state(args->name, STRESS_STATE_WAIT);
 		_exit(0);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -79,30 +84,30 @@ static int stress_kill(const stress_args_t *args)
 
 		t = stress_time_now();
 		ret = kill(args->pid, SIGUSR1);
-		if (ret == 0) {
-			int saved_errno = errno;
+		if (LIKELY(ret == 0)) {
+			const int saved_errno = errno;
 
 			duration += stress_time_now() - t;
 			count += 1.0;
 			errno = saved_errno;
+		} else if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))) {
+			pr_fail("%s: kill PID %" PRIdMAX " with SIGUSR1 failed, errno=%d (%s)\n",
+				args->name, (intmax_t)args->pid, errno, strerror(errno));
 		}
-		if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))
-			pr_fail("%s: kill PID %d with SIGUSR1 failed, errno=%d (%s)\n",
-				args->name, (int)args->pid, errno, strerror(errno));
 
 		/* Zero signal can be used to see if process exists */
 		t = stress_time_now();
 		ret = kill(args->pid, 0);
-		if (ret == 0) {
-			int saved_errno = errno;
+		if (LIKELY(ret == 0)) {
+			const int saved_errno = errno;
 
 			duration += stress_time_now() - t;
 			count += 1.0;
 			errno = saved_errno;
+		} else if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))) {
+			pr_fail("%s: kill PID %" PRIdMAX " with signal 0 failed, errno=%d (%s)\n",
+				args->name, (intmax_t)args->pid, errno, strerror(errno));
 		}
-		if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))
-			pr_fail("%s: kill PID %d with signal 0 failed, errno=%d (%s)\n",
-				args->name, (int)args->pid, errno, strerror(errno));
 
 		/*
 		 * Zero signal can be used to see if process exists,
@@ -111,16 +116,16 @@ static int stress_kill(const stress_args_t *args)
 		 */
 		t = stress_time_now();
 		ret = kill(-1, 0);
-		if (ret == 0) {
-			int saved_errno = errno;
+		if (LIKELY(ret == 0)) {
+			const int saved_errno = errno;
 
 			duration += stress_time_now() - t;
 			count += 1.0;
 			errno = saved_errno;
-		}
-		if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))
+		} else if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))) {
 			pr_fail("%s: kill PID -1 with signal 0 failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+		}
 
 		/*
 		 * Exercise the kernel by sending illegal signal numbers,
@@ -139,7 +144,7 @@ static int stress_kill(const stress_args_t *args)
 		/*
 		 * Send child process some signals to keep it busy
 		 */
-		if (pid > 1) {
+		if (LIKELY(pid > 1)) {
 			VOID_RET(int, kill(pid, 0));
 #if defined(SIGSTOP) && 	\
     defined(SIGCONT)
@@ -152,8 +157,8 @@ static int stress_kill(const stress_args_t *args)
 		bad_pid = stress_get_unused_pid_racy(false);
 		VOID_RET(int, kill(bad_pid, 0));
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -161,16 +166,17 @@ static int stress_kill(const stress_args_t *args)
 		int status;
 
 		VOID_RET(int, kill(pid, SIGKILL));
-		VOID_RET(int, waitpid(pid, &status, 0));
+		VOID_RET(pid_t, waitpid(pid, &status, 0));
 	}
 
 	rate = (duration > 0.0) ? count / duration : 0.0;
-	stress_metrics_set(args, 0, "kill calls per sec", rate);
+	stress_metrics_set(args, 0, "kill calls per sec",
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_kill_info = {
+const stressor_info_t stress_kill_info = {
 	.stressor = stress_kill,
 	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
