@@ -404,8 +404,8 @@ static inline OPTIMIZE3 void stress_memory_contend(const stress_pthread_args_t *
 		read64(data1);
 		stress_asm_mb();
 	}
-	(void)shim_cacheflush((void *)data0, 64, SHIM_DCACHE);
-	(void)shim_cacheflush((void *)data1, 64, SHIM_DCACHE);
+	(void)stress_cpu_data_cache_flush((void *)data0, 64);
+	(void)stress_cpu_data_cache_flush((void *)data1, 64);
 }
 
 /*
@@ -421,7 +421,9 @@ static void *stress_memory_contend_thread(void *arg)
 	 */
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 
-	while (stress_continue_flag()) {
+	stress_random_small_sleep();
+
+	while (LIKELY(stress_continue_flag())) {
 		stress_memory_contend(pa);
 
 #if defined(HAVE_SCHED_SETAFFINITY)
@@ -452,7 +454,7 @@ static int stress_mcontend(stress_args_t *args)
 	void *data[MAX_MAPPINGS];
 	char filename[PATH_MAX];
 	stress_pthread_args_t pa;
-	bool mcontend_numa;
+	bool mcontend_numa = false;
 	int fd, rc;
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	stress_numa_mask_t *numa_mask = NULL;
@@ -476,7 +478,7 @@ static int stress_mcontend(stress_args_t *args)
 
 	fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
 	if (fd < 0) {
-		pr_inf("%s: open failed: errno=%d (%s)\n",
+		pr_inf("%s: open failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)shim_unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
@@ -489,7 +491,7 @@ static int stress_mcontend(stress_args_t *args)
 
 	rc = page_write_sync(fd, page_size);
 	if (rc < 0) {
-		pr_inf("%s: mmap backing file write failed: errno=%d (%s)\n",
+		pr_inf("%s: mmap backing file write failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd);
 		(void)stress_temp_dir_rm_args(args);
@@ -508,7 +510,7 @@ static int stress_mcontend(stress_args_t *args)
 	data[0] = stress_mmap_populate(NULL, page_size, PROT_READ | PROT_WRITE,
 			MAP_PRIVATE, fd, 0);
 	if (data[0] == MAP_FAILED) {
-		pr_inf("%s: mmap failed: errno=%d (%s)\n",
+		pr_inf("%s: mmap failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd);
 		(void)stress_temp_dir_rm_args(args);
@@ -520,7 +522,7 @@ static int stress_mcontend(stress_args_t *args)
 	data[1] = stress_mmap_populate(NULL, page_size , PROT_READ | PROT_WRITE,
 			MAP_PRIVATE, fd, 0);
 	if (data[1] == MAP_FAILED) {
-		pr_inf("%s: mmap failed: errno=%d (%s)\n",
+		pr_inf("%s: mmap failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)munmap(data[0], page_size);
 		(void)close(fd);
