@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2024 Colin Ian King.
+ * Copyright (C) 2021-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,9 +20,14 @@
 #include "stress-ng.h"
 #include "core-cpu.h"
 #include "core-madvise.h"
+#include "core-numa.h"
 #include "core-out-of-memory.h"
 #include "core-target-clones.h"
 
+#if defined(HAVE_LINUX_MEMFD_H)
+#include <linux/memfd.h>
+#endif
+
 #define MIN_MEMFD_BYTES		(2 * MB)
 #define DEFAULT_MEMFD_BYTES	(256 * MB)
 
@@ -36,49 +41,22 @@ static const stress_help_t help[] = {
 	{ NULL,	"memfd N",	 "start N workers allocating memory with memfd_create" },
 	{ NULL,	"memfd-bytes N", "allocate N bytes for each stress iteration" },
 	{ NULL,	"memfd-fds N",	 "number of memory fds to open per stressors" },
+	{ NULL,	"memfd-madvise", "add random madvise hints to memfd mapped pages" },
 	{ NULL,	"memfd-mlock",	 "attempt to mlock pages into memory" },
+	{ NULL,	"memfd-numa",	 "bind memory mappings to randomly selected NUMA nodes" },
 	{ NULL,	"memfd-ops N",	 "stop after N memfd bogo operations" },
+	{ NULL,	"memfd-zap-pte", "enable zap pte bug check (slow)" },
 	{ NULL,	NULL,		 NULL }
 };
 
-static int stress_set_memfd_mlock(const char *opt)
-{
-	return stress_set_setting_true("memfd-mlock", opt);
-}
-
-/*
- *  stress_set_memfd_bytes
- *	set max size of each memfd size
- */
-static int stress_set_memfd_bytes(const char *opt)
-{
-	size_t memfd_bytes;
-
-	memfd_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("memfd-bytes", memfd_bytes,
-		MIN_MEMFD_BYTES, MAX_MEM_LIMIT);
-	return stress_set_setting("memfd-bytes", TYPE_ID_SIZE_T, &memfd_bytes);
-}
-
-/*
- *  stress_set_memfd_fds()
- *      set number of memfd file descriptors
- */
-static int stress_set_memfd_fds(const char *opt)
-{
-	uint32_t memfd_fds;
-
-	memfd_fds = (uint32_t)stress_get_uint64(opt);
-	stress_check_range("memfd-fds", memfd_fds,
-		MIN_MEMFD_FDS, MAX_MEMFD_FDS);
-	return stress_set_setting("memfd-fds", TYPE_ID_UINT32, &memfd_fds);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_memfd_bytes,	stress_set_memfd_bytes },
-	{ OPT_memfd_fds,	stress_set_memfd_fds },
-	{ OPT_memfd_mlock,	stress_set_memfd_mlock },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_memfd_bytes,   "memfd-bytes",   TYPE_ID_SIZE_T_BYTES_VM, MIN_MEMFD_BYTES, MAX_MEM_LIMIT, NULL },
+	{ OPT_memfd_fds,     "memfd-fds",     TYPE_ID_INT32, MIN_MEMFD_FDS, MAX_MEMFD_FDS, NULL },
+	{ OPT_memfd_madvise, "memfd-madvise", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_memfd_mlock,   "memfd-mlock",   TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_memfd_numa,    "memfd-numa",    TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_memfd_zap_pte, "memfd-zap-pte", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT
 };
 
 #if defined(HAVE_MEMFD_CREATE)
@@ -147,39 +125,6 @@ static void stress_memfd_fill_pages_generic(const uint64_t val, void *ptr, const
 	}
 }
 
-#if defined(HAVE_NT_STORE64)
-/*
- *  stress_memfd_fill_pages_nt_store()
- *	fill pages with random uin64_t values using nt_store
- */
-static void stress_memfd_fill_pages_nt_store(const uint64_t val, void *ptr, const size_t size)
-{
-	register uint64_t *u64ptr = (uint64_t *)ptr;
-	register const uint64_t *u64end = uint64_ptr_offset(ptr, size);
-	register uint64_t v = val;
-
-	while (u64ptr < u64end) {
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		stress_nt_store64(u64ptr, v);
-		u64ptr += MEMFD_STRIDE;
-		v++;
-	}
-}
-#endif
-
 #if defined(HAVE_MADVISE) &&	\
     defined(MADV_PAGEOUT)
 /*
@@ -224,28 +169,54 @@ static inline bool stress_memfd_check(
  */
 static int stress_memfd_child(stress_args_t *args, void *context)
 {
-	int *fds, fd;
+	int *fds, rc = EXIT_SUCCESS;
+	register int fd;
 	void **maps;
-	uint64_t i;
+	int32_t i;
 	const size_t page_size = args->page_size;
 	const size_t min_size = 2 * page_size;
-	size_t size;
+	size_t size, flag_index = 0;
 	size_t memfd_bytes = DEFAULT_MEMFD_BYTES;
-	uint32_t memfd_fds = DEFAULT_MEMFD_FDS;
+	int32_t memfd_fds = DEFAULT_MEMFD_FDS;
 	double duration = 0.0, count = 0.0, rate;
+	bool memfd_madvise = false;
 	bool memfd_mlock = false;
-#if defined(HAVE_NT_STORE64)
-	void (*stress_memfd_fill_pages)(uint64_t val, void *ptr, const size_t size) =
-		stress_cpu_x86_has_sse2() ? stress_memfd_fill_pages_nt_store : stress_memfd_fill_pages_generic;
-#else
-	void (*stress_memfd_fill_pages)(uint64_t val, void *ptr, const size_t size) = stress_memfd_fill_pages_generic;
+	bool memfd_numa = false;
+	bool memfd_zap_pte = false;
+	char filename_rndstr[64], filename_unusual[64], filename_pid[64];
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+	stress_numa_mask_t *numa_mask = NULL;
 #endif
 
 	stress_catch_sigill();
 
 	(void)context;
 
+	(void)stress_get_setting("memfd-madvise", &memfd_madvise);
 	(void)stress_get_setting("memfd-mlock", &memfd_mlock);
+	(void)stress_get_setting("memfd-numa", &memfd_numa);
+	(void)stress_get_setting("memfd-zap-pte", &memfd_zap_pte);
+
+#if !defined(HAVE_MADVISE)
+	if (memfd_madvise) {
+		if (args->instance == 0) {
+			pr_inf("%s: disabling --memfd-madvise, madvise() "
+				"not supported\n", args->name);
+		}
+		memfd_madvise = false;
+	}
+#endif
+
+#if !defined(HAVE_MADVISE) ||	\
+    !defined(MADV_PAGEOUT)
+	if (memfd_zap_pte) {
+		if (args->instance == 0) {
+			pr_inf("%s: disabling --memfd-zap-pte, madvise() "
+				"with MADV_PAGEOUT not supported\n", args->name);
+		}
+		memfd_zap_pte = false;
+	}
+#endif
 
 	if (!stress_get_setting("memfd-bytes", &memfd_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -253,7 +224,7 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			memfd_bytes = MIN_MEMFD_BYTES;
 	}
-	memfd_bytes /= args->num_instances;
+	memfd_bytes /= args->instances;
 	if (memfd_bytes < MIN_MEMFD_BYTES)
 		memfd_bytes = MIN_MEMFD_BYTES;
 
@@ -263,25 +234,59 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 	if (size < min_size)
 		size = min_size;
 
-	fds = calloc(memfd_fds, sizeof(*fds));
+	fds = (int *)calloc(memfd_fds, sizeof(*fds));
 	if (!fds) {
-		pr_inf("%s: cannot allocate fds buffer: %d (%s)\n",
+		pr_inf("%s: cannot allocate fds buffer, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
-	maps = calloc(memfd_fds, sizeof(*maps));
+	maps = (void **)calloc(memfd_fds, sizeof(*maps));
 	if (!maps) {
-		pr_inf("%s: cannot allocate maps buffer: %d (%s)\n",
+		pr_inf("%s: cannot allocate maps buffer, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		free(fds);
 		return EXIT_NO_RESOURCE;
 	}
 
+	if (memfd_numa) {
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+		if (stress_numa_nodes() > 1) {
+			numa_mask = stress_numa_mask_alloc();
+			if (!numa_mask) {
+				pr_inf("%s: cannot allocate NUMA mask, disabling --memfd-numa\n",
+					args->name);
+				memfd_numa = false;
+			}
+		} else {
+			if (args->instance == 0) {
+				pr_inf("%s: only 1 NUMA node available, disabling --memfd-numa\n",
+					args->name);
+				memfd_numa = false;
+			}
+		}
+#else
+		if (args->instance == 0)
+			pr_inf("%s: --memfd-numa selected but not supported by this system, disabling option\n",
+				args->name);
+		memfd_numa = false;
+#endif
+	}
+
+	stress_rndstr(filename_rndstr, sizeof(filename_rndstr));
+	(void)snprintf(filename_unusual, sizeof(filename_unusual),
+		"memfd-%c[H%c%c:?*~", 27, 7, 255);
+	(void)snprintf(filename_pid, sizeof(filename_pid),
+		"memfd-%" PRIdMAX "-%" PRIu64, (intmax_t)args->pid, stress_mwc64());
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		char filename[PATH_MAX];
+		char filename[64];
 		double t;
+		int min_fd = INT_MAX;
+		int max_fd = INT_MIN;
 
 		for (i = 0; i < memfd_fds; i++) {
 			fds[i] = -1;
@@ -294,14 +299,19 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 				break;
 
 			(void)snprintf(filename, sizeof(filename),
-				"memfd-%" PRIdMAX "-%" PRIu64,
+				"memfd-%" PRIdMAX "-%" PRId32,
 				(intmax_t)args->pid, i);
 
 			t = stress_time_now();
-			fds[i] = shim_memfd_create(filename, 0);
-			if (fds[i] >= 0) {
+			fd = shim_memfd_create(filename, 0);
+			if (fd >= 0) {
 				duration += stress_time_now() - t;
 				count += 1.0;
+				if (min_fd > fd)
+					min_fd = fd;
+				if (max_fd < fd)
+					max_fd = fd;
+				fds[i] = fd;
 			} else {
 				switch (errno) {
 				case EMFILE:
@@ -312,13 +322,14 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 				case ENOSYS:
 				case EFAULT:
 				default:
-					pr_fail("%s: memfd_create failed: errno=%d (%s)\n",
+					pr_fail("%s: memfd_create failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					stress_continue_set_flag(false);
+					rc = EXIT_FAILURE;
 					goto memfd_unmap;
 				}
 			}
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				goto memfd_unmap;
 		}
 
@@ -333,34 +344,40 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 			if (fds[i] < 0)
 				continue;
 
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				break;
 
 			/* Allocate space */
 			ret = ftruncate(fds[i], (off_t)size);
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				switch (errno) {
 				case EINTR:
 					break;
 				default:
 					pr_fail("%s: ftruncate failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 					break;
 				}
 			}
 			/*
-			 *  ..and map it in, using MAP_POPULATE
-			 *  to force page it in
+			 *  ..and map it in, don't populate as this is expensive
 			 */
-			maps[i] = stress_mmap_populate(NULL, size, PROT_WRITE,
+			maps[i] = mmap(NULL, size, PROT_WRITE,
 					MAP_FILE | MAP_SHARED, fds[i], 0);
-			if (maps[i] == MAP_FAILED)
+			if (UNLIKELY(maps[i] == MAP_FAILED))
 				continue;
 			if (memfd_mlock)
 				(void)shim_mlock(maps[i], size);
-			stress_memfd_fill_pages(stress_mwc64(), maps[i], size);
-			(void)stress_madvise_random(maps[i], size);
-			(void)stress_madvise_mergeable(maps[i], size);
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+			if (memfd_numa && numa_mask)
+				stress_numa_randomize_pages(numa_mask, maps[i], size, page_size);
+#endif
+			stress_memfd_fill_pages_generic(stress_mwc64(), maps[i], size);
+			if (memfd_madvise) {
+				(void)stress_madvise_random(maps[i], size);
+				(void)stress_madvise_mergeable(maps[i], size);
+			}
 
 #if defined(FALLOC_FL_PUNCH_HOLE) &&	\
     defined(FALLOC_FL_KEEP_SIZE)
@@ -378,7 +395,7 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 			 */
 			VOID_RET(ssize_t, shim_fallocate(fds[i], 0, (off_t)size, 0));
 
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				goto memfd_unmap;
 		}
 
@@ -389,40 +406,50 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 				continue;
 #if defined(SEEK_SET)
 			if (lseek(fds[i], (off_t)size >> 1, SEEK_SET) < 0) {
-				if ((errno != ENXIO) && (errno != EINVAL))
+				if (UNLIKELY((errno != ENXIO) && (errno != EINVAL))) {
 					pr_fail("%s: lseek SEEK_SET failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(SEEK_CUR)
 			if (lseek(fds[i], (off_t)0, SEEK_CUR) < 0) {
-				if ((errno != ENXIO) && (errno != EINVAL))
+				if (UNLIKELY((errno != ENXIO) && (errno != EINVAL))) {
 					pr_fail("%s: lseek SEEK_CUR failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(SEEK_END)
 			if (lseek(fds[i], (off_t)0, SEEK_END) < 0) {
-				if ((errno != ENXIO) && (errno != EINVAL))
+				if (UNLIKELY((errno != ENXIO) && (errno != EINVAL))) {
 					pr_fail("%s: lseek SEEK_END failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(SEEK_HOLE)
 			if (lseek(fds[i], (off_t)0, SEEK_HOLE) < 0) {
-				if ((errno != ENXIO) && (errno != EINVAL))
+				if (UNLIKELY((errno != ENXIO) && (errno != EINVAL))) {
 					pr_fail("%s: lseek SEEK_HOLE failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(SEEK_DATA)
 			if (lseek(fds[i], (off_t)0, SEEK_DATA) < 0) {
-				if ((errno != ENXIO) && (errno != EINVAL))
+				if (UNLIKELY((errno != ENXIO) && (errno != EINVAL))) {
 					pr_fail("%s: lseek SEEK_DATA failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
-			if (!stress_continue_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				goto memfd_unmap;
 		}
 
@@ -433,57 +460,62 @@ memfd_unmap:
 		}
 #if defined(HAVE_MADVISE) &&	\
     defined(MADV_PAGEOUT)
-		/*
-		 *  Check for zap_pte bug, see Linux commit
-		 *  5abfd71d936a8aefd9f9ccd299dea7a164a5d455
-		 */
-		for (i = 0; stress_continue_flag() && (i < memfd_fds); i++) {
-			uint64_t *buf;
-			uint64_t val;
-			const ssize_t test_size = page_size << 1;
+		if (UNLIKELY(memfd_zap_pte)) {
+			/*
+			 *  Check for zap_pte bug, see Linux commit
+			 *  5abfd71d936a8aefd9f9ccd299dea7a164a5d455
+			 */
+			for (i = 0; LIKELY(stress_continue_flag() && (i < memfd_fds)); i++) {
+				uint64_t *buf;
+				uint64_t val;
+				const ssize_t test_size = page_size << 1;
 
-			if (ftruncate(fds[i], (off_t)test_size) < 0)
-				continue;
-			buf = mmap(NULL, test_size, PROT_READ | PROT_WRITE,
-					MAP_PRIVATE, fds[i], 0);
-			if (buf == MAP_FAILED)
-				continue;
-			if (memfd_mlock)
-				(void)shim_mlock(buf, test_size);
-			val = stress_mwc64();
-			stress_memfd_fill_pages(val, buf, test_size);
+				if (UNLIKELY(ftruncate(fds[i], (off_t)test_size) < 0))
+					continue;
+				buf = mmap(NULL, test_size, PROT_READ | PROT_WRITE,
+						MAP_PRIVATE, fds[i], 0);
+				if (UNLIKELY(buf == MAP_FAILED))
+					continue;
+				if (memfd_mlock)
+					(void)shim_mlock(buf, test_size);
+				val = stress_mwc64();
+				stress_memfd_fill_pages_generic(val, buf, test_size);
 
-			if (madvise(buf, test_size, MADV_PAGEOUT) < 0)
-				goto buf_unmap;
-			if (ftruncate(fds[i], (off_t)page_size) < 0)
-				goto buf_unmap;
-			if (ftruncate(fds[i], (off_t)test_size) < 0)
-				goto buf_unmap;
-			if (!stress_memfd_check(val, buf, page_size, 1))
-				pr_fail("%s: unexpected memfd %d data mismatch in first page\n",
-					args->name, fds[i]);
-			if (!stress_memfd_check(0ULL, uint64_ptr_offset(buf, page_size), page_size, 0))
-				pr_fail("%s: unexpected memfd %d data mismatch in zero'd second page\n",
-					args->name, fds[i]);
+				if (UNLIKELY(madvise(buf, test_size, MADV_PAGEOUT) < 0))
+					goto buf_unmap;
+				if (UNLIKELY(ftruncate(fds[i], (off_t)page_size) < 0))
+					goto buf_unmap;
+				if (UNLIKELY(ftruncate(fds[i], (off_t)test_size) < 0))
+					goto buf_unmap;
+				if (UNLIKELY(!stress_memfd_check(val, buf, page_size, 1))) {
+					pr_fail("%s: unexpected memfd %d data mismatch in first page\n",
+						args->name, fds[i]);
+					rc = EXIT_FAILURE;
+				}
+				if (UNLIKELY(!stress_memfd_check(0ULL, uint64_ptr_offset(buf, page_size), page_size, 0))) {
+					pr_fail("%s: unexpected memfd %d data mismatch in zero'd second page\n",
+						args->name, fds[i]);
+					rc = EXIT_FAILURE;
+				}
 buf_unmap:
-			(void)munmap((void *)buf, test_size);
-			VOID_RET(int, ftruncate(fds[i], 0));
+				(void)munmap((void *)buf, test_size);
+				VOID_RET(int, ftruncate(fds[i], 0));
+			}
 		}
 #endif
-
-		stress_close_fds(fds, memfd_fds);
+		if (shim_close_range(min_fd, max_fd, 0) < 0) {
+			for (i = 0; i < memfd_fds; i++) {
+				(void)close(fds[i]);
+			}
+		}
 
 		/* Exercise illegal memfd name */
-		stress_rndstr(filename, sizeof(filename));
-		fd = shim_memfd_create(filename, 0);
+		fd = shim_memfd_create(filename_rndstr, 0);
 		if (fd >= 0)
 			(void)close(fd);
 
 		/* Exercise illegal flags */
-		(void)snprintf(filename, sizeof(filename),
-			"memfd-%" PRIdMAX "-%" PRIu64,
-			(intmax_t)args->pid, stress_mwc64());
-		fd = shim_memfd_create(filename, ~0U);
+		fd = shim_memfd_create(filename_pid, ~0U);
 		if (fd >= 0)
 			(void)close(fd);
 
@@ -498,9 +530,7 @@ buf_unmap:
 			(void)close(fd);
 
 		/* Exercise unusual chars in name */
-		(void)snprintf(filename, sizeof(filename),
-			"memfd-%c[H%c%c:?*~", 27, 7, 255);
-		fd = shim_memfd_create(filename, 0);
+		fd = shim_memfd_create(filename_unusual, 0);
 		if (fd >= 0)
 			(void)close(fd);
 
@@ -514,19 +544,17 @@ buf_unmap:
 		if (fd >= 0)
 			(void)close(fd);
 
-		/* Exercise all flags */
-		for (i = 0; i < (uint64_t)SIZEOF_ARRAY(flags); i++) {
-			(void)snprintf(filename, sizeof(filename),
-				"memfd-%" PRIdMAX"-%" PRIu64,
-				(intmax_t)args->pid, i);
-			t = stress_time_now();
-			fd = shim_memfd_create(filename, flags[i]);
-			if (fd >= 0) {
-				duration += stress_time_now() - t;
-				count += 1.0;
-				(void)close(fd);
-			}
+		/* Exercise next flag */
+		t = stress_time_now();
+		fd = shim_memfd_create(filename_pid, flags[flag_index]);
+		if (fd >= 0) {
+			duration += stress_time_now() - t;
+			count += 1.0;
+			(void)close(fd);
 		}
+		flag_index++;
+		if (flag_index >= SIZEOF_ARRAY(flags))
+			flag_index = 0;
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
@@ -534,12 +562,16 @@ buf_unmap:
 
 	rate = (count > 0.0) ? duration / count : 0.0;
 	stress_metrics_set(args, 0, "nanosecs per memfd_create call",
-		rate * STRESS_DBL_NANOSECOND, STRESS_HARMONIC_MEAN);
+		rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
 
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+	if (numa_mask)
+		stress_numa_mask_free(numa_mask);
+#endif
 	free(maps);
 	free(fds);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
@@ -551,18 +583,18 @@ static int stress_memfd(stress_args_t *args)
 	return stress_oomable_child(args, NULL, stress_memfd_child, STRESS_OOMABLE_NORMAL);
 }
 
-stressor_info_t stress_memfd_info = {
+const stressor_info_t stress_memfd_info = {
 	.stressor = stress_memfd,
 	.class = CLASS_OS | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_memfd_info = {
+const stressor_info_t stress_memfd_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without memfd_create() system call"
