@@ -146,7 +146,7 @@ static int stress_mmapfork(stress_args_t *args)
 			s_pids[i].pid = -1;
 
 		for (i = 0; i < MAX_PIDS; i++) {
-			if (!stress_continue(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto reap;
 
 			s_pids[i].pid = fork();
@@ -164,13 +164,13 @@ static int stress_mmapfork(stress_args_t *args)
 					_exit(MMAPFORK_FAILURE);
 
 				(void)shim_memset(&info, 0, sizeof(info));
-				if (sysinfo(&info) < 0) {
+				if (UNLIKELY(sysinfo(&info) < 0)) {
 					pr_fail("%s: sysinfo failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					_exit(MMAPFORK_FAILURE);
 				}
 
-				len = ((size_t)info.freeram / (args->num_instances * MAX_PIDS)) / 2;
+				len = ((size_t)info.freeram / (args->instances * MAX_PIDS)) / 2;
 
 #if defined(MADV_WIPEONFORK)
 				if (wipe_ok && (wipe_ptr != MAP_FAILED) &&
@@ -184,6 +184,8 @@ static int stress_mmapfork(stress_args_t *args)
 				if (!stress_get_setting("mmapfork-bytes", &len)) {
 					if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 						len = MIN_MMAPFORK_BYTES;
+					if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+						len = MAX_32;
 				}
 
 				if (len < MIN_MMAPFORK_BYTES)
@@ -230,9 +232,9 @@ static int stress_mmapfork(stress_args_t *args)
 				continue;
 
 			if (shim_waitpid(s_pids[i].pid, &status, 0) < 0) {
-				if (errno != EINTR) {
-					pr_err("%s: waitpid errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+				if (UNLIKELY(errno != EINTR)) {
+					pr_err("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)s_pids[i].pid, errno, strerror(errno));
 				} else {
 					/* Probably an SIGARLM, force reap */
 					goto reap;
