@@ -175,7 +175,7 @@ case_sched_fifo:
 		    (errno != ENOSYS) &&
 		    (errno != EBUSY)) {
 			pr_dbg("%s: sched_setscheduler "
-				"failed: errno=%d (%s) "
+				"failed, errno=%d (%s) "
 				"for scheduler policy %s\n",
 				args->name, errno, strerror(errno),
 				policy_name);
@@ -193,7 +193,7 @@ static int stress_yield(stress_args_t *args)
 	size_t metrics_size;
 	uint64_t max_ops_per_yielder;
 	int32_t cpus = stress_get_processors_configured();
-	const uint32_t instances = args->num_instances;
+	const uint32_t instances = args->instances;
 	uint32_t yielders = 2, yield_procs = 0;
 	double count, duration, ns;
 #if defined(HAVE_SCHED_GETAFFINITY)
@@ -202,7 +202,12 @@ static int stress_yield(stress_args_t *args)
 	pid_t *pids;
 	size_t i, yield_sched = SIZE_MAX;
 
-	(void)stress_get_setting("yield-procs", &yield_procs);
+	if (!stress_get_setting("yield-procs", &yield_procs)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			yield_procs = MAX_YIELD_PROCS;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			yield_procs = MIN_YIELD_PROCS;
+	}
 	(void)stress_get_setting("yield-sched", &yield_sched);
 
 #if defined(HAVE_SCHED_GETAFFINITY)
@@ -245,7 +250,7 @@ static int stress_yield(stress_args_t *args)
 	} else {
 		yielders = yield_procs;
 	}
-	max_ops_per_yielder = args->max_ops / yielders;
+	max_ops_per_yielder = args->bogo.max_ops / yielders;
 
 	pids = (pid_t *)calloc(yielders, sizeof(*pids));
 	if (!pids) {
@@ -272,11 +277,11 @@ static int stress_yield(stress_args_t *args)
 	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	for (i = 0; stress_continue_flag() && (i < yielders); i++) {
+	for (i = 0; LIKELY(stress_continue_flag() && (i < yielders)); i++) {
 		pids[i] = fork();
 		if (pids[i] < 0) {
 			pr_dbg("%s: fork failed (instance %" PRIu32
-				", yielder %zd): errno=%d (%s)\n",
+				", yielder %zd), errno=%d (%s)\n",
 				args->name, args->instance, i, errno, strerror(errno));
 		} else if (pids[i] == 0) {
 			stress_parent_died_alarm();
